!---------------------------------------------------------------------------------------
! PREP_FLUXES: prep_flux
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> prep_flux
!!
!! Purpose:    Averages/interpolates fluxes to the FLEXPART grid and saves these
!!             in a standardized format.
!!
!---------------------------------------------------------------------------------------

program prep_flux

  use mod_var
  use mod_settings
  use mod_dates

  implicit none

  type (settings_t)                   :: settings
  character(len=max_path_len)         :: file_settings
  real, dimension(:), allocatable     :: time, time_out
  real, dimension(:), allocatable     :: lon_in, lat_in, lon_out, lat_out
  real, dimension(:,:,:), allocatable :: flx_in, flx_tmp, flx_out
  integer                             :: nt

  ! read file settings
  ! ------------------

  call getarg(1, file_settings)
  if (file_settings == '') then
    write(*,*) 'ERROR: need to specify settings file'
    stop
  endif

  call read_file_settings(file_settings, settings)

  call init_settings(settings)

  allocate( time(ntime) )
  allocate( lon_in(nx_in) )
  allocate( lat_in(ny_in) )
  allocate( flx_in(nx_in, ny_in, ntime) )
  allocate( flx_tmp(nx_in, ny_in, ntime_out) )
  allocate( time_out(ntime) )
  allocate( lon_out(nx_out) )
  allocate( lat_out(ny_out) )
  allocate( flx_out(nx_out, ny_out, ntime_out) )

  ! read input fluxes
  ! -----------------

  call read_ncdf(settings%file_in, settings%varname_in, settings%lonname_in, settings%latname_in, &
                  settings%timename_in, settings%indextime, time, lon_in, lat_in, flx_in)


  ! monthly variability
  ! -------------------

  if( len_trim(settings%file_scalar).ne.0 ) then
    call calc_monthly(settings, flx_in, flx_tmp)
  else
    if( ntime.eq.ntime_out ) then
      flx_tmp = flx_in
    else if( ntime.eq.1 ) then
      do nt = 1, ntime_out
        flx_tmp(:,:,nt) = flx_in(:,:,1)
      end do
    else
      write(*,*) 'ERROR: incompatible ntime and ntime_out'
      stop
    endif
  endif 
 
  ! convert fluxes to output grid 
  ! -----------------------------

  call convertgrid(settings%midpoints, lon_in, lat_in, flx_tmp, &
                     lon_out, lat_out, flx_out)

  ! convert to output units
  ! -----------------------

  flx_out = flx_out * settings%coeff

  ! convert time to days since 1-Jan-1900
  ! -------------------------------------

  call adjust_time(settings, time, time_out)

  ! write output fluxes to ncdf 
  ! --------------------------- 

  call write_ncdf(settings%file_out, settings%varname_out, settings%lonname_out, settings%latname_out, &
                   settings%timename_out, settings%varunit, time_out, lon_out, lat_out, flx_out)

  
end program prep_flux


