!---------------------------------------------------------------------------------------
! PREP_SATELLITE: get_tropomi_blnd
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2022, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> get_tropomi_blnd
!!
!! Purpose:    Reads observations and auxilliary data from the specified satellite and
!!             and writes the output into files that can be directly used by FLEXPART
!!             and, for inversions, by FLEXINVERT
!!
!! Interface:
!!
!!    Inputs
!!             settings  -  settings data structure
!!
!!    Outputs
!!
!!    Externals
!!
!---------------------------------------------------------------------------------------

subroutine get_tropomi_blnd(settings)

  use mod_settings
  use mod_var
  use mod_dates
  use mod_save
  use netcdf

  implicit none

  type (settings_t), intent(in)                         :: settings

  character(max_name_len), dimension(:), allocatable    :: filelist, filereadlist
  character(max_name_len)                               :: line, filename
  character(len=27), dimension(:), allocatable          :: timestr
  character(len=8)                                      :: adate
  character(len=6)                                      :: atime
  character(len=3)                                      :: aspec
  character(len=6), dimension(4)                        :: units
  integer                                               :: i, ierr, n, nf, nl, nm, np, nt, nbits, iswater 
  integer                                               :: nfiles, nread, nretr, npx_in_scan, numbox
  real(kind=8)                                          :: jdatei, jdatef, jd, days_since_ref, jdateref, jdmeas
  real                                                  :: sec_since_midnight
  integer                                               :: jjjjmmdd, hhmiss, jjjj, mm, dd, hh, mi, ss
  integer, parameter                                    :: refdate = 19700101
  integer, parameter                                    :: lsynctime = 600
  integer                                               :: ntime, nmeas, ncoord, nlayer
  real, dimension(:,:), allocatable                     :: lat_bnds, lon_bnds
  real, dimension(:), allocatable                       :: vmr, verr, time, qa, albedo
  real, dimension(:,:), allocatable                     :: vapri, vdryair, cak
  real, dimension(:), allocatable                       :: psurf, pdel
  real, dimension(:,:), allocatable                     :: xpoints, ypoints
  real, dimension(:,:), allocatable                     :: zpoint1, zpoint2
  real, dimension(:,:), allocatable                     :: cak_out, vapri_out, vdryair_out
  real, dimension(:,:), allocatable                     :: cak_tmp, vapri_tmp, vdryair_tmp
  real, dimension(:), allocatable                       :: vmr_out, verr_out, albedo_out
  real, dimension(:), allocatable                       :: cpri, cakpri, cdryair
  integer, dimension(:), allocatable                    :: idate, itime, surfclass
  character(len=8)                                      :: surfclass_bin, qa_bin 
  integer                                               :: specnum, nchar
  integer                                               :: ncid, dimid, varid
  real                                                  :: nest_llx, nest_lly, nest_urx, nest_ury
  real                                                  :: llx, lly, urx, ury
  integer                                               :: nargs, cnt
  character(max_name_len), dimension(2)                 :: args
  character(len=28)                                     :: str
  real                                                  :: sclfact, offset
  character(len=2)                                      :: anread

  ! get species number
  ! ------------------

  open(100,file=trim(settings%path_flexpart)//'options/SPECIES/spec_overview',status='old',action='read',iostat=ierr)
  nchar = len_trim(settings%species)
  do while ( ierr.eq.0 )
    read (100, fmt='(A)', iostat=ierr) line
    if ( line(len_trim(line)-nchar+1:len_trim(line)) == trim(settings%species) ) ierr = 1
  end do
  read(line(9:11),*) specnum
  write(logid,*) 'species: ',settings%species
  write(logid,*) 'specnum: ',specnum
  write(logid,*) 'qa cutoff: ',settings%qa_cutoff
  close(100)

  ! define domains
  ! --------------------

  llx = settings%outlonleft
  lly = settings%outlatlower
  urx = llx + settings%numxgrid*settings%dxout
  ury = lly + settings%numygrid*settings%dyout

  if ( settings%lnested.eq.1 ) then
    nest_llx = settings%outlonnest
    nest_lly = settings%outlatnest
    nest_urx = nest_llx + settings%numxnest*settings%dxoutnest
    nest_ury = nest_lly + settings%numynest*settings%dyoutnest
  endif
  print*, 'llx,lly,urx,ury = ',llx,lly,urx,ury
  if (settings%lnested.eq.1) print*, 'nest_llx,nest_lly,nest_urx,nest_ury = ',nest_llx,nest_lly,nest_urx,nest_ury

  ! list satellite files
  ! ----------------------

  call system('find '//trim(settings%path_obs)//' -type f | grep '//trim(settings%suffix)//&
              ' | wc -l > '//trim(settings%path_obsout)//'obsfiles.txt')
  call system('find '//trim(settings%path_obs)//' -type f | grep '//trim(settings%suffix)//&
              ' >> '//trim(settings%path_obsout)//'obsfiles.txt')

  open(100,file=trim(settings%path_obsout)//'obsfiles.txt',action='read',status='old',iostat=ierr)
  if ( ierr.ne. 0 ) then
    write(logid,*) 'ERROR: cannot open obsfiles.txt'
    stop
  endif
  read(100,*,iostat=ierr) nfiles
  allocate ( filelist(nfiles), stat = ierr )
  allocate ( filereadlist(nfiles), stat = ierr )
  do nf = 1, nfiles
    read(100,fmt='(A)',iostat=ierr) filelist(nf)
    if (ierr.ne.0) exit
  end do
  close(100)

  ! loop over days 
  ! --------------

  jdatei = juldate(settings%datei, 0)
  jdatef = juldate(settings%datef, 0)
  jd = jdatei

  read_loop: do

    if ( jd.gt.jdatef ) exit read_loop

    ! search files for current date
    call caldate(jd, jjjjmmdd, hhmiss)
    write(adate,fmt='(I8.8)') jjjjmmdd
    write(logid,*) 'current day: ',adate
    nread = 0
    do n = 1, nfiles
      do i = 1, len_trim(filelist(n))-18
        if ( filelist(n)(i:(i+7)).eq.adate ) then
          nread = nread + 1
          filereadlist(nread) = filelist(n)
          exit
        endif
      end do
    end do
    write(logid,*) 'number of files for current day: ',nread
    if ( nread.eq.0 ) then 
      jd = jd + 1d0
      cycle read_loop     
    endif

    ! loop over files for current day
    ! -------------------------------

    do n = 1, nread

      ! open file
      write(logid,*) 'reading file: ',filereadlist(n)
      call check( nf90_open(trim(filereadlist(n)),nf90_NOWRITE,ncid) )

      ! dimension variables
      call check( nf90_inq_dimid(ncid,trim(settings%nmeas_name),dimid) )
      call check( nf90_inquire_dimension(ncid,dimid,len=nmeas) )
      write(logid,*) 'number of retrievals: ',nmeas
      if ( nmeas.eq.0 ) then
        write(logid,*) 'file contains no retrievals moving to next file'
        cycle
      endif
      call check( nf90_inq_dimid(ncid,trim(settings%ncoord_name),dimid) )
      call check( nf90_inquire_dimension(ncid,dimid,len=ncoord) )
      write(logid,*) 'number of geocoordinates: ',ncoord
      call check( nf90_inq_dimid(ncid,trim(settings%nlevel_name),dimid) )
      call check( nf90_inquire_dimension(ncid,dimid,len=nlayer) )
      write(logid,*) 'number of vertical layers: ',nlayer

      ! geolocation variables
      allocate( lat_bnds(ncoord,nmeas), stat=ierr )
      if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate lat_bnds'
      call check( nf90_inq_varid(ncid,trim(settings%lat_name),varid) )
      call check( nf90_get_var(ncid,varid,lat_bnds) )
      allocate( lon_bnds(ncoord,nmeas), stat=ierr )
      if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate lon_bnds'
      call check( nf90_inq_varid(ncid,trim(settings%lon_name),varid) )
      call check( nf90_get_var(ncid,varid,lon_bnds) )
      
      ! product variables
      allocate( timestr(nmeas), stat=ierr )
      if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate timestr'
      ! cannot read variable type string in fortran
!      call check( nf90_inq_varid(ncid,trim(settings%time_name),varid) )
!      call check( nf90_get_var(ncid,varid,timestr) )
      ! work around -> extract time variable to ascii
      call system("ncdump -v "//trim(settings%time_name)//" "//trim(filereadlist(n))//&
                  " | sed -e '1,/data:/d' -e '$d' | tail -n +2 > "//trim(settings%path_obsout)//"time.txt")
      open(100,file=trim(settings%path_obsout)//"time.txt",action='read',status='old',iostat=ierr)
      if (ierr.ne.0) then
        write(logid,*) 'ERROR: cannot open time.txt'
        stop
      endif
      cnt=0
      do while (ierr.eq.0)
        read (100, fmt='(A)', iostat=ierr) line
        if (ierr.ne.0) exit
        ! remove pesky characters in first line
        if (cnt==0) then
          line = line(12:len(line))
        endif
        call parse_string(line, ',', args, nargs)
        do i = 1, nargs
          cnt=cnt+1
          str=trim(args(i))  
          timestr(cnt)=str(2:28)
        end do
      end do
      close(100)
      call system("rm -f "//trim(settings%path_obsout)//"time.txt")
      allocate( psurf(nmeas), stat=ierr )
      if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate psurf'
      call check( nf90_inq_varid(ncid,trim(settings%psurf_name),varid) )
      call check( nf90_get_var(ncid,varid,psurf) )
      allocate( pdel(nmeas), stat=ierr )
      if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate pdel'
      call check( nf90_inq_varid(ncid,trim(settings%pdel_name),varid) )
      call check( nf90_get_var(ncid,varid,pdel) )
      allocate( qa(nmeas), stat=ierr )
      if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate qa'
      call check( nf90_inq_varid(ncid,trim(settings%qa_name),varid) )
      call check( nf90_get_var(ncid,varid,qa) )
      call check( nf90_get_att(ncid,varid,'scale_factor',sclfact) )
      call check( nf90_get_att(ncid,varid,'add_offset',offset) )
      qa = qa*sclfact + offset
      print*, 'qa = ',qa(1:20)
      allocate( vmr(nmeas), stat=ierr )
      if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate vmr'
      call check( nf90_inq_varid(ncid,trim(settings%vmr_name),varid) )
      call check( nf90_get_var(ncid,varid,vmr) )
      allocate( verr(nmeas), stat=ierr )
      if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate verr'
      call check( nf90_inq_varid(ncid,trim(settings%verr_name),varid) )
      call check( nf90_get_var(ncid,varid,verr) )

      ! detailed results variables
      allocate( cak(nlayer,nmeas), stat=ierr )
      if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate cak'
      call check( nf90_inq_varid(ncid,trim(settings%ak_name),varid) )
      call check( nf90_get_var(ncid,varid,cak) )
      allocate( vapri(nlayer,nmeas), stat=ierr )
      if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate vapri'
      call check( nf90_inq_varid(ncid,trim(settings%apri_name),varid) )
      call check( nf90_get_var(ncid,varid,vapri) )
      allocate( vdryair(nlayer,nmeas), stat=ierr )
      if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate vdryair'
      call check( nf90_inq_varid(ncid,trim(settings%dryair_name),varid) )
      call check( nf90_get_var(ncid,varid,vdryair) )
      allocate( surfclass(nmeas), stat=ierr ) 
      if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate surfclass'
      call check( nf90_inq_varid(ncid,trim(settings%surfclass_name),varid) )
      call check( nf90_get_var(ncid,varid,surfclass) )
      allocate( albedo(nmeas), stat=ierr )
      if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate albedo'
      call check( nf90_inq_varid(ncid,trim(settings%albedo_name),varid) )
      call check( nf90_get_var(ncid,varid,albedo) )

      call check( nf90_close(ncid)) 

      ! select retrievals and extract needed info
      if( n.eq.1 ) then
        allocate( xpoints(maxretr,ncoord), stat=ierr )
        if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate xpoints'
        allocate( ypoints(maxretr,ncoord), stat=ierr )
        if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate ypoints'
        allocate( zpoint1(maxretr,nlayer), stat=ierr )
        if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate zpoint1'
        allocate( zpoint2(maxretr,nlayer), stat=ierr )
        if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate zpoint2'
        allocate( idate(maxretr), stat=ierr )
        if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate idate'
        allocate( itime(maxretr), stat=ierr )
        if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate itime'
        allocate( cak_out(maxretr,nlayer), stat=ierr )
        if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate cak_out'
        allocate( vapri_out(maxretr,nlayer), stat=ierr )
        if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate vapri_out'
        allocate( vdryair_out(maxretr,nlayer), stat=ierr )
        if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate vdryair_out'
        allocate( vmr_out(maxretr), stat=ierr )
        if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate vmr_out'
        allocate( verr_out(maxretr), stat=ierr )
        if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate verr_out'
        allocate( albedo_out(maxretr), stat=ierr )
        if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate albedo_out'
        nretr = 0
      endif
      print*, 'nmeas = ',nmeas
      do nm = 1, nmeas
        ! exclude retrievals above quality cutoff
        if ( qa(nm).lt.settings%qa_cutoff ) cycle
        write(surfclass_bin, fmt='(B0)') surfclass(nm)
        nbits = len_trim(surfclass_bin)
        read(surfclass_bin(nbits:nbits),*) iswater
        if(nm.lt.10) print*, 'surfclass_bin, nbits, iswater = ',surfclass_bin, nbits, iswater
        ! if landonly exclude pixels over water
        if ( settings%landonly.and.iswater.eq.1 ) cycle
        if ( settings%lnested.eq.1 ) then
          ! if nested domain exclude pixels outside
          if ( any(lon_bnds(:,nm).lt.nest_llx).or. &
               any(lon_bnds(:,nm).gt.nest_urx).or. &
               any(lat_bnds(:,nm).lt.nest_lly).or. &
               any(lat_bnds(:,nm).gt.nest_ury) ) cycle
        else
          ! if not nested exclude pixels outside global domain
          if ( any(lon_bnds(:,nm).lt.llx).or. &
               any(lon_bnds(:,nm).gt.urx).or. &
               any(lat_bnds(:,nm).lt.lly).or. &
               any(lat_bnds(:,nm).gt.ury) ) cycle
        endif
        read(timestr(nm)(1:4),'(I4)') jjjj
        read(timestr(nm)(6:7),'(I2)') mm
        read(timestr(nm)(9:10),'(I2)') dd
        read(timestr(nm)(12:13),'(I2)') hh
        read(timestr(nm)(15:16),'(I2)') mi
        read(timestr(nm)(18:19),'(I2)') ss
        hhmiss=hh*10000+mi*100+ss
        jjjjmmdd=jjjj*10000+mm*100+dd
        jdmeas=juldate(jjjjmmdd,hhmiss)
        ! exclude retrievals if not for current day
        if ( floor(jdmeas).ne.jd ) cycle
        ! check date and time
        if (isnan(real(hhmiss)).or.(jjjjmmdd.lt.0)) then
          write(logid,*) 'WARNING: problem with itime, idate for file, nmeas, jdmeas = ',trim(filereadlist(n)), nm, jdmeas
          cycle
        endif
        nretr = nretr + 1
        if (nretr.lt.10) print*, 'jjjjmmdd, hhmiss = ',jjjjmmdd, hhmiss
        idate(nretr) = jjjjmmdd
        itime(nretr) = hhmiss
        ! corners ordered anticlockwise from lower left
        xpoints(nretr,:) = lon_bnds(:,nm)
        ypoints(nretr,:) = lat_bnds(:,nm)
        ! pressure levels are equipressure apart
        zpoint1(nretr,1) = psurf(nm)
        zpoint2(nretr,1) = psurf(nm) - pdel(nm)
        do nl = 2, nlayer
          zpoint1(nretr,nl) = zpoint1(nretr,nl-1) - pdel(nm)
          zpoint2(nretr,nl) = zpoint2(nretr,nl-1) - pdel(nm)
        end do
        cak_out(nretr,:) = cak(:,nm)
        vapri_out(nretr,:) = vapri(:,nm)
        vdryair_out(nretr,:) = vdryair(:,nm)
        vmr_out(nretr) = vmr(nm)
        verr_out(nretr) = verr(nm)
        albedo_out(nretr) = albedo(nm)
      end do

      write(logid,*) 'number of retrievals: ',nretr

      if ( allocated(lat_bnds) )      deallocate(lat_bnds)
      if ( allocated(lon_bnds) )      deallocate(lon_bnds)
      if ( allocated(vapri) )         deallocate(vapri)
      if ( allocated(vdryair) )       deallocate(vdryair)
      if ( allocated(cak) )           deallocate(cak)
      if ( allocated(timestr) )       deallocate(timestr)
      if ( allocated(psurf) )         deallocate(psurf)
      if ( allocated(pdel) )          deallocate(pdel)
      if ( allocated(qa) )            deallocate(qa)
      if ( allocated(vmr) )           deallocate(vmr)
      if ( allocated(verr) )          deallocate(verr)
      if ( allocated(surfclass) )     deallocate(surfclass)
      if ( allocated(albedo) )        deallocate(albedo)

    end do  ! nread

    if ( nretr.eq.0 ) go to 10

    ! reverse direction of vertical coordinate
    ! raw data ordered from top of atmosphere to surface
    ! which is opposite to the pressure levels calculated (zpoint)
    allocate( cak_tmp(maxretr,nlayer), stat=ierr )
    if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate cak_tmp'
    allocate( vapri_tmp(maxretr,nlayer), stat=ierr )
    if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate vapri_tmp'
    allocate( vdryair_tmp(maxretr,nlayer), stat=ierr )
    if ( ierr.ne.0 ) write(logid,*) 'ERROR: cannot allocate vdryair_tmp'
    do nl = 1, nlayer
      cak_tmp(:,nl) = cak_out(:,(nlayer-nl+1))
      vapri_tmp(:,nl) = vapri_out(:,(nlayer-nl+1))
      vdryair_tmp(:,nl) = vdryair_out(:,(nlayer-nl+1))
    end do
    cak_out = cak_tmp
    vapri_out = vapri_tmp
    vdryair_out = vdryair_tmp
    deallocate( cak_tmp, vapri_tmp, vdryair_tmp)

    ! calculate column prior (units mol/m2)
    allocate( cpri(maxretr), stat=ierr )
    cpri(1:nretr) = sum(vapri_out(1:nretr,:), dim=2)

    ! calculate column prior convolved with averaging kernel (units mol/m2) 
    allocate( cakpri(maxretr), stat=ierr )
    do nm = 1, nretr
      cakpri(nm) = dot_product(vapri_out(nm,:), cak_out(nm,:))
    end do

    ! calculate column dry air (units mol/m2)
    allocate( cdryair(maxretr), stat=ierr )
    cdryair(1:nretr) = sum(vdryair_out(1:nretr,:), dim=2)

    ! average pixels
    ! --------------

    if ( settings%avg_pixels ) then
      ! nretr is number of individual observations
      ! after averaging numbox is number of super observations
      call average(settings, idate, itime, xpoints, ypoints, zpoint1, zpoint2, vmr_out, verr_out, &
                   vapri_out, cpri, cakpri, cak_out, cdryair, vdryair_out, albedo_out, nretr, &
                   ncoord, nlayer, numbox)
    else
      ! no averaging
      numbox = nretr
    endif 

    ! write output data
    ! -----------------

    ! adjust itime 
    ! needed so that with flexpart rounding to lsynctime all retrievals are still for current day
    do nm = 1, numbox
      if ( itime(nm).gt.(235959-lsynctime*100/60) ) then
        itime(nm) = 235950-lsynctime*100/60
      endif
    end do

    ! write command file
    call prep_command(settings,jd)

    ! write outgrid file
    call prep_outgrid(settings,jd)

    ! write ageclasses file
    call prep_ageclass(settings,jd)

    ! write pathnames file
    call prep_pathnames(settings,jd)

    ! units of output data: dryair, cdryair, cpri, vmr
    units(1) = 'mol/m2'
    units(2) = 'mol/m2'
    units(3) = 'mol/m2'
    units(4) = 'ppbv'

    ! write releases data for flexpart
    call write_releases(settings,numbox,nlayer,ncoord,xpoints,ypoints,zpoint1,zpoint2,&
                        idate,itime,cak_out,vdryair_out,specnum,units)

    ! write retrieval data for flexinvert
    call write_retrieval(settings,numbox,ncoord,xpoints,ypoints,idate,itime,&
                         cpri,cakpri,vmr_out,verr_out,cdryair,albedo_out,units)

    ! copy standard input files to options folder
    write(aspec,fmt='(I3.3)') specnum
    call system('mkdir -p '//trim(settings%path_options)//adate//'/options/SPECIES/')
    filename = trim(settings%path_flexpart)//'options/SPECIES/SPECIES_'//aspec
    call system('cp '//trim(filename)//' '//trim(settings%path_options)//adate//'/options/SPECIES/')
    call system('cp '//trim(settings%path_flexpart)//'options/*.dat'//' '//trim(settings%path_options)//adate//'/options/')
    call system('cp '//trim(settings%path_flexpart)//'options/*.t'//' '//trim(settings%path_options)//adate//'/options/')

    ! no retrievals on this day
10 continue

    ! prepare for next day
    ! -------------------- 

    if ( allocated(xpoints) )     deallocate(xpoints)
    if ( allocated(ypoints) )     deallocate(ypoints)
    if ( allocated(zpoint1) )     deallocate(zpoint1)
    if ( allocated(zpoint2) )     deallocate(zpoint2)
    if ( allocated(idate) )       deallocate(idate)
    if ( allocated(itime) )       deallocate(itime)
    if ( allocated(cak_out) )     deallocate(cak_out)
    if ( allocated(vapri_out) )   deallocate(vapri_out)
    if ( allocated(vdryair_out) ) deallocate(vdryair_out)
    if ( allocated(vmr_out) )     deallocate(vmr_out)
    if ( allocated(verr_out) )    deallocate(verr_out)
    if ( allocated(cpri) )        deallocate(cpri)
    if ( allocated(cakpri) )      deallocate(cakpri)
    if ( allocated(cdryair) )     deallocate(cdryair)
    if ( allocated(albedo_out) )  deallocate(albedo_out)

    ! iterate over days
    jd = jd + 1d0

  end do read_loop

  if ( allocated(filelist) )     deallocate(filelist)
  if ( allocated(filereadlist) ) deallocate(filereadlist)


end subroutine get_tropomi_blnd

